/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import kotlinx.android.parcel.Parcelize

@Parcelize
open class PickerRequestParams(
    open val title: String,
    open val pickerFriendType: PickerFriendType? = PickerFriendType.KAKAO_TALK,
    open val pickerFriendFilter: PickerFriendFilter? = PickerFriendFilter.NONE,
    open val viewAppearance: ViewAppearance? = ViewAppearance.AUTO,
    open val countryCodes: List<String>? = null,
    open val usingOsFilter: OsFilter? = OsFilter.ALL,
    open val orientation: PickerOrientation? = PickerOrientation.AUTO,
    open val enableSearch: Boolean? = true,
    open val searchPlaceHolder: String? = null,
    open val enableIndex: Boolean? = true,
    open val showMyProfile: Boolean? = true,
    open val showFavorite: Boolean? = true,
    open val disableSelectReasons: DisableSelectionReason? = null
) : Parcelable

@Parcelize
class MultiPickerRequestParams(
    override val title: String,
    override val pickerFriendType: PickerFriendType? = PickerFriendType.KAKAO_TALK,
    override val pickerFriendFilter: PickerFriendFilter? = PickerFriendFilter.NONE,
    override val viewAppearance: ViewAppearance? = ViewAppearance.AUTO,
    override val countryCodes: List<String>? = null,
    override val usingOsFilter: OsFilter? = OsFilter.ALL,
    override val orientation: PickerOrientation? = PickerOrientation.AUTO,
    override val enableSearch: Boolean? = true,
    override val searchPlaceHolder: String? = null,
    override val enableIndex: Boolean? = true,
    override val showMyProfile: Boolean? = true,
    override val showFavorite: Boolean? = true,
    override val disableSelectReasons: DisableSelectionReason? = null,
    val maxPickableCount: Int? = 30,
    val minPickableCount: Int? = 1,
    val showPickedFriend: Boolean? = true
) : PickerRequestParams(
    title,
    pickerFriendType,
    pickerFriendFilter,
    viewAppearance,
    countryCodes,
    usingOsFilter,
    orientation,
    enableSearch,
    searchPlaceHolder,
    enableIndex,
    showMyProfile,
    showFavorite,
    disableSelectReasons
)

/**
 * 친구를 가져올 서비스 선택
 */
enum class PickerFriendType {
    /**
     * 카카오톡 친구
     */
    @SerializedName("talk")
    KAKAO_TALK,
    /**
     * 카카오스토리 친구
     */
    @SerializedName("story")
    KAKAO_STORY,
    /**
     * 카카오톡과 카카오스토리 친구
     */
    @SerializedName("talkstory")
    KAKAO_TALK_AND_STORY,
    @UnknownValue
    UNKNOWN;
}

enum class PickerOrientation {
    PORTRAIT, LANDSCAPE, AUTO
}

enum class PickerFriendFilter {
    REGISTERED, INVITABLE, NONE
}

enum class ViewAppearance {
    LIGHT, DARK, AUTO
}

enum class OsFilter {
    ALL, ANDROID, IOS
}

enum class DisableSelectionReason {
    MSG_BLOCKED, ETC
}